@@ -1,5 +1,4 @@
-/* $Id: rbldnsd.c,v 1.117 2006-11-24 16:35:28 mjt Exp $
- * rbldnsd: main program
+/* rbldnsd: main program
  */
 
 #define _LARGEFILE64_SOURCE /* to define O_LARGEFILE if supported */
@@ -121,6 +120,8 @@ const struct dstype *ds_types[] = {
   dstype(ip4set),
   dstype(ip4tset),
   dstype(ip4trie),
+  dstype(ip6tset),
+  dstype(ip6trie),
   dstype(dnset),
 #ifdef DNHASH
   dstype(dnhash),
