@@ -1,5 +1,4 @@
-/* $Id: rbldnsd.h,v 1.97 2006/02/01 10:03:25 mjt Exp $
- * common rbldnsd #include header
+/* common rbldnsd #include header
  */
 
 #include <stdarg.h>
@@ -7,6 +6,7 @@
 #include <stdio.h>
 #include "config.h"
 #include "ip4addr.h"
+#include "ip6addr.h"
 #include "dns.h"
 #include "mempool.h"
 
@@ -62,8 +62,10 @@ struct dnsqinfo {	/* qi */
   unsigned qi_tflag;			/* query RR type flag (NSQUERY_XX) */
   unsigned qi_dnlen0;			/* length of qi_dn - 1 */
   unsigned qi_dnlab;			/* number of labels in q_dn */
-  ip4addr_t qi_ip4;			/* parsed IP4 address */
   int qi_ip4valid;			/* true if qi_ip4 is valid */
+  int qi_ip6valid;			/* true if qi_ip6 is valid */
+  ip4addr_t qi_ip4;			/* parsed IP4 address */
+  ip6oct_t qi_ip6[IP6ADDR_FULL];	/* parsed IP6 address */
 };
 
 #define PACK32(b,n) ((b)[0]=(n)>>24,(b)[1]=(n)>>16,(b)[2]=(n)>>8,(b)[3]=(n))
@@ -90,6 +92,7 @@ char *parse_dn(char *s, unsigned char *dn, unsigned *dnlenp);
  * readdslines() */
 int parse_a_txt(char *str, const char **rrp, const char *def_rr,
                 struct dsctx *dsc);
+char *firstword_lc(char *line, const char *word_lc);
 
 typedef void ds_startfn_t(struct dataset *ds);
 typedef int ds_linefn_t(struct dataset *ds, char *line, struct dsctx *dsc);
@@ -140,6 +143,7 @@ struct dstype {	/* dst */
 
 /* dst_flags */
 #define DSTF_IP4REV	0x01	/* ip4 set */
+#define DSTF_IP6REV	0x02	/* ip6 set */
 #define DSTF_SPECIAL	0x08	/* special ds: non-recursive */
 
 #define declaredstype(t) extern const struct dstype dataset_##t##_type
@@ -160,6 +164,8 @@ struct dstype {	/* dst */
 declaredstype(ip4set);
 declaredstype(ip4tset);
 declaredstype(ip4trie);
+declaredstype(ip6tset);
+declaredstype(ip6trie);
 declaredstype(dnset);
 declaredstype(dnhash);
 declaredstype(generic);
@@ -214,7 +220,8 @@ struct dataset {	/* ds */
 #define DSF_NSWARN 0x02			/* warned about new-style NS */
 #endif
   unsigned ds_ttl;			/* default ttl for a dataset */
-  char *ds_subst[10];			/* substitution variables */
+  char *ds_subst[11];			/* substitution variables */
+#define SUBST_BASE_TEMPLATE	10
   struct mempool *ds_mp;		/* memory pool for data */
   struct dataset *ds_next;		/* next in global list */
 };
@@ -270,6 +277,9 @@ struct zone {	/* zone, list of zones */
 #ifndef NO_STATS
   struct dnsstats z_stats;		/* statistic counters */
   struct dnsstats z_pstats;		/* for stats monitoring: prev values */
+#endif
+#ifndef NO_DSO
+  void *z_hookdata;			/* data ptr for hooks */
 #endif
   struct zone *z_next;			/* next in list */
 };
@@ -305,12 +315,16 @@ void addrr_any(struct dnspacket *pkt, unsigned dtp,
 int ds_acl_query(const struct dataset *ds, struct dnspacket *pkt);
 
 #ifndef NO_MASTER_DUMP
-void dump_a_txt(const char *name, const unsigned char *rr,
+void dump_a_txt(const char *name, const char *rr,
                 const char *subst, const struct dataset *ds, FILE *f);
 void dump_ip4range(ip4addr_t a, ip4addr_t b, const char *rr,
 		   const struct dataset *ds, FILE *f);
 void dump_ip4(ip4addr_t a, const char *rr, const struct dataset *ds, FILE *f);
 void dumpzone(const struct zone *z, FILE *f);
+void dump_ip6(const ip6oct_t *addr, unsigned wild_nibbles, const char *rr,
+              const struct dataset *ds, FILE *f);
+void dump_ip6range(const ip6oct_t *beg, const ip6oct_t *end, const char *rr,
+                   const struct dataset *ds, FILE *f);
 #endif
 #define TXTBUFSIZ 260
 int txtsubst(char txtbuf[TXTBUFSIZ], const char *template,
@@ -412,22 +426,37 @@ int PRINTFLIKE(3, 4) ssprintf(char *buf, int bufsz, const char *fmt, ...);
       ) \
   )
 
-struct ip4trie_node {	/* IPv4 trie node */
-  ip4addr_t ip4t_prefix;		/* high bits of address prefix */
-  ip4addr_t ip4t_bits;			/* number of bits in prefix */
-  struct ip4trie_node *ip4t_right;	/* if (bit+1) bit is set, go here.. */
-  struct ip4trie_node *ip4t_left;	/* ..or else here. */
-  const char *ip4t_data;		/* data of the node if any */
-};
+/* hooks from a DSO extensions */
+#ifndef NO_DSO
 
-struct ip4trie {
-  struct ip4trie_node *ip4t_root;	/* root node of the tree */
-  unsigned ip4t_nents;			/* number of entries so far */
-  unsigned ip4t_nnodes;			/* total number of nodes in tree */
-};
+/* prototype for init routine */
+int rbldnsd_extension_init(char *arg, struct zone *zonelist);
 
-const char *ip4trie_lookup(const struct ip4trie *trie, ip4addr_t q);
+/* return true/false depending whenever hook needs to be reloaded */
+extern int (*hook_reload_check)(const struct zone *zonelist);
 
-struct ip4trie_node *
-ip4trie_addnode(struct ip4trie *trie, ip4addr_t prefix, unsigned bits,
-                struct mempool *mp);
+/* perform actual reload, after all zones has been reloaded. */
+extern int (*hook_reload)(struct zone *zonelist);
+
+/* check whenever this query is allowed for this client:
+ *  * 0 = ok, <0 = drop the packet, >0 = refuse */
+extern int (*hook_query_access)
+  (const struct sockaddr *requestor,
+   const struct zone *zone,
+   const struct dnsqinfo *qinfo);
+
+/* notice result of the OK query */
+extern int (*hook_query_result)
+  (const struct sockaddr *requestor,
+   const struct zone *zone,
+   const struct dnsqinfo *qinfo,
+   int positive);
+
+#define call_hook(name, args)	\
+	(hook_##name ? hook_##name args : 0)
+
+#else	/* dummy "functions" */
+
+#define call_hook(name, args) 0
+
+#endif
