@@ -1,5 +1,4 @@
-/* $Id: rbldnsd_packet.c,v 1.97 2005/12/19 14:36:21 mjt Exp $
- * DNS packet handling routines for rbldnsd
+/* DNS packet handling routines for rbldnsd
  */
 
 #include <string.h>
@@ -8,10 +7,10 @@
 #include <sys/types.h>
 #include <unistd.h>
 #include <netinet/in.h>
+#include <arpa/inet.h>
 #include <netdb.h>
 #include <syslog.h>
 #include "rbldnsd.h"
-#include "rbldnsd_hooks.h"
 
 #ifndef NO_IPv6
 # ifndef NI_MAXHOST
@@ -159,48 +158,11 @@ parsequery(struct dnspacket *pkt, unsigned qlen,
   return 1;
 }
 
-#ifdef RECOGNIZE_IP4IN6
-static const unsigned char *const ip6p =
-  "\001f\001f\001f\001f\0010\0010\0010\0010"
-  "\0010\0010\0010\0010\0010\0010\0010\0010"
-  "\0010\0010\0010\0010\0010\0010\0010\0010"
-  "\003ip6";
-#endif
-
-/* parse DN (as in 4.3.2.1.in-addr.arpa) to ip4addr_t */
-static int
-dntoip4addr(const unsigned char *q,
-            unsigned UNUSED qlen0, unsigned qlab,
-            ip4addr_t *ap) {
-  ip4addr_t a = 0, o;
-  if (qlab != 4) {
-
-#ifdef RECOGNIZE_IP4IN6
-
-     if ((qlab != 33 || qlen0 != 68 || memcmp(q + 16, ip6p, 28) != 0) &&
-         (qlab != 32 || qlen0 != 64 || memcmp(q + 16, ip6p, 24) != 0))
-       return 0;
-     for (o = 0; o < 32; o += 4) {
-       ++q;
-       if (*q >= '0' && *q <= '9')
-         a |= (unsigned)(*q++ - '0') << o;
-       else if (*q >= 'a' && *q <= 'f')
-         a |= (unsigned)(*q++ - 'a' + 10) << o;
-       else
-         return 0;
-     }
-     *ap = a;
-     return 1;
-
-#else /* RECOGNIZE_IP4IN6 */
-     return 0;
-#endif
-
-  }
-
 #define digit(c) ((c) >= '0' && (c) <= '9')
 #define d2n(c) ((unsigned)((c) - '0'))
 
+static int dntoip4addr(const unsigned char *q, ip4addr_t *ap) {
+  unsigned o, a = 0;
 #define oct(q,o)					\
     switch(*q) {					\
     case 1:						\
@@ -228,8 +190,57 @@ dntoip4addr(const unsigned char *q,
   *ap = a;
   return 1;
 #undef oct
-#undef digit
-#undef d2n
+}
+
+static int dntoip6addr(const unsigned char *q, ip6oct_t ap[IP6ADDR_FULL]) {
+  unsigned o1, o2, c;
+  for(c = IP6ADDR_FULL; c; ) {
+    if (*q++ != 1)
+      return 0;
+    if (digit(*q)) o2 = d2n(*q++);
+    else if (*q >= 'a' && *q <= 'f') o2 = *q++ - 'a' + 10;
+    else return 0;
+    if (*q++ != 1)
+      return 0;
+    if (digit(*q)) o1 = d2n(*q++);
+    else if (*q >= 'a' && *q <= 'f') o1 = *q++ - 'a' + 10;
+    else return 0;
+    ap[--c] = (o1 << 4) | o2;
+  }
+  return 1;
+}
+
+static const ip6oct_t ip6mapped_pfx[12] =
+  "\0\0\0\0\0\0\0\0"
+  "\377\377\377\377";
+
+/* parse DN (as in 4.3.2.1.in-addr.arpa) to ip4addr_t (4 octets 0..255).
+ * parse DN (as in 0.1.2.3.4.5...f.base.dn) to ip6 address (32 nibbles 0..f)
+ */
+static void dntoip(struct dnsqinfo *qi, int flags) {
+
+  const unsigned char *q = qi->qi_dn;
+  unsigned qlab = qi->qi_dnlab;
+
+  qi->qi_ip4valid = qlab == 4 && dntoip4addr(q, &qi->qi_ip4);
+  if (qi->qi_ip4valid) {
+    if (flags & DSTF_IP6REV) {
+      /* construct IP4MAPPED address */
+      memcpy(qi->qi_ip6, ip6mapped_pfx, sizeof(ip6mapped_pfx));
+      PACK32(qi->qi_ip6 + sizeof(ip6mapped_pfx), qi->qi_ip4);
+      qi->qi_ip6valid = 1;
+    }
+  }
+  else {
+    qi->qi_ip6valid =
+      qlab == 32 && qi->qi_dnlen0 == 64 && dntoip6addr(q, qi->qi_ip6);
+    if (flags & DSTF_IP4REV &&
+	memcmp(qi->qi_ip6, ip6mapped_pfx, sizeof(ip6mapped_pfx)) == 0) {
+      /* construct IP4 from IP4MAPPED */
+      qi->qi_ip4 = unpack32(qi->qi_ip6 + sizeof(ip6mapped_pfx));
+      qi->qi_ip4valid = 1;
+    }
+  }
 }
 
 const struct zone *
@@ -249,9 +260,9 @@ findqzone(const struct zone *zone,
   qi->qi_dnlptr = dnlptr;
   qi->qi_dnlab = dnlab - zone->z_dnlab;
   qi->qi_dnlen0 = dnlen - zone->z_dnlen;
-  if (zone->z_dstflags & DSTF_IP4REV) /* ip4 address */
-    qi->qi_ip4valid =
-      dntoip4addr(qi->qi_dn, qi->qi_dnlen0, qi->qi_dnlab, &qi->qi_ip4);
+
+  if (zone->z_dstflags & (DSTF_IP4REV|DSTF_IP6REV)) /* IP address */
+    dntoip(qi, zone->z_dstflags);
 
   return zone;
 }
@@ -362,12 +373,10 @@ int replypacket(struct dnspacket *pkt, unsigned qlen, struct zone *zone) {
   if (qi.qi_tflag & NSQUERY_REFUSE)
     refuse(DNS_R_REFUSED);
 
-#ifdef do_hook_query_access
-  if ((found = hook_query_access(zone, NULL, &qi))) {
+  if ((found = call_hook(query_access, (pkt->p_peer, zone, &qi)))) {
     if (found < 0) return 0;
     refuse(DNS_R_REFUSED);
   }
-#endif
 
   if (qi.qi_dnlab == 0) {	/* query to base zone: SOA and NS */
 
@@ -414,9 +423,6 @@ int replypacket(struct dnspacket *pkt, unsigned qlen, struct zone *zone) {
     addrr_soa(pkt, zone, 1);	/* add SOA if any to AUTHORITY */
     h[p_f2] = DNS_R_NXDOMAIN;
     do_stats(zone->z_stats.q_nxd += 1);
-#ifdef do_hook_query_result
-    hook_query_result(zone, NULL, &qi, 0);
-#endif
   }
   else {
     if (!h[p_ancnt2]) {	/* positive reply, no answers */
@@ -427,10 +433,8 @@ int replypacket(struct dnspacket *pkt, unsigned qlen, struct zone *zone) {
              !lazy)
       addrr_ns(pkt, zone, 1); /* add nameserver records to positive reply */
     do_stats(zone->z_stats.q_ok += 1);
-#ifdef do_hook_query_result
-    hook_query_result(zone, NULL, &qi, 1);
-#endif
   }
+  (void)call_hook(query_result, (pkt->p_peer, zone, &qi, found));
   if (rlen() > DNS_MAXPACKET) {	/* add OPT record for long replies */
     /* as per parsequery(), we always have 11 bytes for minimal OPT record at
      * the end of our reply packet, OR rlen() does not exceed DNS_MAXPACKET */
@@ -934,11 +938,13 @@ addrr_a_txt(struct dnspacket *pkt, unsigned qtflag,
             const struct dataset *ds) {
   if (qtflag & NSQUERY_A)
     addrr_any(pkt, DNS_T_A, rr, 4, ds->ds_ttl);
-  if (rr[4] && (qtflag & NSQUERY_TXT)) {
-    char sb[TXTBUFSIZ];
+  if (qtflag & NSQUERY_TXT) {
+    char sb[TXTBUFSIZ+1];
     unsigned sl = txtsubst(sb + 1, rr + 4, subst, ds);
-    sb[0] = sl;
-    addrr_any(pkt, DNS_T_TXT, sb, sl + 1, ds->ds_ttl);
+    if (sl) {
+      sb[0] = sl;
+      addrr_any(pkt, DNS_T_TXT, sb, sl + 1, ds->ds_ttl);
+    }
   }
 }
 
@@ -982,7 +988,7 @@ void logreply(const struct dnspacket *pkt, FILE *flog, int flushlog) {
   else
     *cp++ = '?';
 #else
-  strcpy(cp, inet_ntoa(((struct sockaddr_in*)pkt->p_peer)->sin_addr.s_addr));
+  strcpy(cp, inet_ntoa(((struct sockaddr_in*)pkt->p_peer)->sin_addr));
   cp += strlen(cp);
 #endif
   *cp++ = ' ';
@@ -991,7 +997,7 @@ void logreply(const struct dnspacket *pkt, FILE *flog, int flushlog) {
       dns_typename(((unsigned)q[0]<<8)|q[1]),
       dns_classname(((unsigned)q[2]<<8)|q[3]),
       dns_rcodename(pkt->p_buf[p_f2] & pf2_rcode),
-      pkt->p_buf[p_ancnt2], pkt->p_cur - pkt->p_buf);
+      pkt->p_buf[p_ancnt2], (int)(pkt->p_cur - pkt->p_buf));
   if (flushlog)
     write(fileno(flog), cbuf, cp - cbuf);
   else
