@@ -1,5 +1,4 @@
-/* $Id: rbldnsd_zones.c,v 1.69 2006/02/14 13:47:34 mjt Exp $
- * Nameserver zones: structures and routines
+/* Nameserver zones: structures and routines
  */
 
 #include <stdio.h>
@@ -133,6 +132,7 @@ struct zone *addzone(struct zone *zonelist, const char *spec) {
   dnlen = dns_ptodn(name, dn, sizeof(dn));
   if (!dnlen)
     error(0, "invalid domain name `%.80s'", name);
+  dns_dntol(dn, dn);
 
   p = estrdup(p+1);
   ds = newdataset(p);
@@ -159,24 +159,11 @@ struct zone *addzone(struct zone *zonelist, const char *spec) {
   return zonelist;
 }
 
-/* helper routine for ds_special */
-static char *firstword(char *line, const char *word_lc) {
-  while(*word_lc)
-    if (dns_dnlc(*line) != *word_lc)
-      return NULL;
-    else
-      ++word_lc, ++line;
-  if (!ISSPACE(*line))
-    return NULL;
-  SKIPSPACE(line);
-  return line;
-}
-
 /* parse $SPECIAL construct */
 static int ds_special(struct dataset *ds, char *line, struct dsctx *dsc) {
   char *w;
 
-  if ((w = firstword(line, "soa"))) {
+  if ((w = firstword_lc(line, "soa"))) {
     /* SOA record */
     struct dssoa dssoa;
     unsigned char odn[DNS_MAXDN], pdn[DNS_MAXDN];
@@ -206,8 +193,8 @@ static int ds_special(struct dataset *ds, char *line, struct dsctx *dsc) {
     return 1;
   }
 
-  if ((w = firstword(line, "ns")) ||
-      (w = firstword(line, "nameserver"))) {
+  if ((w = firstword_lc(line, "ns")) ||
+      (w = firstword_lc(line, "nameserver"))) {
      /* NS records */
      unsigned char dn[DNS_MAXDN];
      unsigned dnlen;
@@ -215,7 +202,10 @@ static int ds_special(struct dataset *ds, char *line, struct dsctx *dsc) {
      unsigned ttl;
 
 #ifndef INCOMPAT_0_99
+#ifdef __GNUC__
+/* some compilers don't understand #warning directive */
 #warning NS record compatibility mode: remove for 1.0 final
+#endif
      struct dsns *dsns_first = 0;
      unsigned cnt;
      int newformat = 0;
@@ -285,7 +275,7 @@ static int ds_special(struct dataset *ds, char *line, struct dsctx *dsc) {
     return 1;
   }
 
-  if ((w = firstword(line, "ttl"))) {
+  if ((w = firstword_lc(line, "ttl"))) {
     unsigned ttl;
     if (!(w = parse_ttl(w, &ttl, def_ttl))) return 0;
     if (*w) return 0;
@@ -294,7 +284,7 @@ static int ds_special(struct dataset *ds, char *line, struct dsctx *dsc) {
     return 1;
   }
 
-  if ((w = firstword(line, "maxrange4"))) {
+  if ((w = firstword_lc(line, "maxrange4"))) {
     unsigned r;
     int cidr;
     if (*w == '/') cidr = 1, ++w;
@@ -313,9 +303,9 @@ static int ds_special(struct dataset *ds, char *line, struct dsctx *dsc) {
     return 1;
   }
 
-  if (*(w = line) >= '0' && *w <= '9' && ISSPACE(w[1])) {
+  if (((*(w = line) >= '0' && *w <= '9') || *w == '=') && ISSPACE(w[1])) {
     /* substitution vars */
-    unsigned n = w[0] - '0';
+    unsigned n = w[0] == '=' ? SUBST_BASE_TEMPLATE : w[0] - '0';
     if (dsc->dsc_subset) ds = dsc->dsc_subset;
     if (ds->ds_subst[n]) return 1; /* ignore second assignment */
     w += 2;
@@ -325,13 +315,13 @@ static int ds_special(struct dataset *ds, char *line, struct dsctx *dsc) {
     return 1;
   }
 
-  if ((w = firstword(line, "dataset"))) {
+  if ((w = firstword_lc(line, "dataset"))) {
     if (!isdstype(ds->ds_type, combined))
       return 0;	/* $dataset is only allowed for combined dataset */
     return ds_combined_newset(ds, w, dsc);
   }
 
-  if ((w = firstword(line, "timestamp"))) {
+  if ((w = firstword_lc(line, "timestamp"))) {
     time_t stamp, expires;
 
     if (!(w = parse_timestamp(w, &stamp))) return 0;
@@ -342,7 +332,7 @@ static int ds_special(struct dataset *ds, char *line, struct dsctx *dsc) {
       if (!(w = parse_time(w + 1, &n)) || *w) return 0;
       if (!stamp || !n) return 0;
       expires = stamp + n;
-      if (expires < 0 || expires - n != stamp) return 0;
+      if (expires < 0 || expires - (time_t)n != stamp) return 0;
     }
     else {
       if (!(w = parse_timestamp(w, &expires)) || *w) return 0;
@@ -420,6 +410,7 @@ static void freedataset(struct dataset *ds) {
   ds->ds_ttl = def_ttl;
   ds->ds_dsns = NULL;
   ds->ds_nsttl = 0;
+  ds->ds_expires = 0;
 #ifndef INCOMPAT_0_99
   ds->ds_nsflags = 0;
 #endif
@@ -528,7 +519,7 @@ void dumpzone(const struct zone *z, FILE *f) {
   const struct dslist *dsl;
   { /* zone header */
     char name[DNS_MAXDOMAIN+1];
-    const unsigned char **nsdna = z->z_nsdna;
+    const unsigned char *const *nsdna = z->z_nsdna;
     const struct dssoa *dssoa = z->z_dssoa;
     unsigned nns = z->z_nns;
     unsigned n;
